@@ -1,40 +1,45 @@
 ddelap <-
   function (x, alpha, beta, lambda, log = FALSE) {
-    DDLAP <- vector(length = length(x), mode = "numeric")
+    DDLAP <- double(length(x))
     DDLAP <- ddelap_C(x, alpha, beta, lambda, log)
     return(DDLAP)
   }
+
 pdelap <-
   function (q, alpha, beta, lambda, lower.tail = TRUE, log.p = FALSE) {
-    PDLAP <- vector(length = length(q), mode = "numeric")
+    PDLAP <- double(length(q))
     PDLAP <- pdelap_C(q, alpha, beta, lambda, lower.tail, log.p)
     return(PDLAP)
   }
+
 qdelap <-
   function (p, alpha, beta, lambda, lower.tail = TRUE, log.p = FALSE, exact = TRUE) {
-    QDLAP <- vector(length = length(p), mode = "numeric")
+    QDLAP <- double(length(p))
     if (exact) {
       QDLAP <- qdelap_C(p, alpha, beta, lambda, lower.tail, log.p)
     } else {
+      if (log.p) p <- exp(p)
+      if (!lower.tail) p <- 1 - p
       pValid <- p[p > 0 & p < 1]
       pNan <- p[p < 0]
       p0 <- p[p == 0]
       pInf <- p[p >= 1]
-      n <- min(10 ^ (ceiling(log(alpha * beta + lambda, 10)) + 3), 1e7)
+      n <- min(10 ^ (ceiling(log(alpha * beta + lambda, 10)) + 5), 1e7)
       NB <- rnbinom(n, mu = alpha * beta, size = alpha)
       P <- rpois(n, lambda = lambda)
       DP <- NB + P
-      QValid <- as.vector(quantile(DP, pValid, na.rm = TRUE))
+      QValid <- as.vector(quantile(DP, pValid, na.rm = TRUE, type = 8))
       QNan <- rep.int(NaN, times = length(pNan))
       Q0 <- rep.int(0, times = length(p0))
       QInf <- rep.int(Inf, times = length(pInf))
-      QDLAP <- as.vector(c(QNan, Q0, QValid, QInf))
+      QDLAP <- as.vector(c(QNan, Q0, QValid, QInf), mode = 'integer')
     }  
     return(QDLAP)
   }
+
 rdelap <-
   function (n, alpha, beta, lambda, exact = TRUE) {
-    RDLAP <- vector(length = length(n), mode = "numeric")
+    RDLAP <- double(length(n))
     if (exact) {
       RDLAP <- rdelap_C(n, alpha, beta, lambda)
     } else {
@@ -51,7 +56,7 @@ rdelap <-
   }
 
 MoMdelap <- function (x) {
-    MoMDLAP <- vector(length = 3, mode = "numeric")
+    MoMDLAP <- double(3)
     MoMDLAP <- MoMdelap_C(x)
     if (any(MoMDLAP < 0)) stop ("Method of moments not appropriate for this data; results include negative parameters.")
     return(MoMDLAP)
